
bridge =  (require "./Bridge.coffee").bridge
utils =  (require "./Utils.coffee")


{app} = (require "./App.coffee")

{TableComponent} = require "./TableComponent.coffee"



Framer.Defaults.Layer =
	backgroundColor: ""

Colors =
	background: "#151619"
	text: "#6A7281"
	textHighlight: "#FFFFFF"
	textInvisible: "#3B4250"
	indent: "#2A2D33"


class InputComponent extends Layer
	constructor: (options) ->
		super options
		@inputElement = document.createElement("input")
		_.extend @inputElement.style,
			backgroundColor: "transparent"
			border: "0px"
			margin: "0px"
			padding: "0px"

		@_updateSize()
		@_element.appendChild(@inputElement)

		@on("change:width", @_updateSize)
		@on("change:height", @_updateSize)

	_updateSize: =>
		_.extend @inputElement.style,
			marginTop: "-1px"
			marginLeft: "-1px"
			width: "#{@width - 2}px"
			height: "#{@height - 2}px"

	@define "value",
		get: -> @inputElement.value
		set: (value) -> @inputElement.value = value
		


# Sizes = {}
# Sizes.base = 8
# Sizes.rowHeight = 36
# Sizes.indent = Sizes.base
# Sizes.indentWidth = 1
# Sizes.indentHeight = Sizes.base * 2

Sizes = {}
Sizes.base = 8
Sizes.rowHeight = 32
Sizes.indent = 12
Sizes.indentWidth = 1
Sizes.indentHeight = Sizes.rowHeight

grey = (n) -> "rgb(#{n}, #{n}, #{n})"

scroll = new Layer
	x: 180 + 180
	width: 180
	height: window.innerHeight

Screen.on "resize", ->
	scroll.height = window.innerHeight

scroll.scroll = true
scroll.scrollHorizontal = false
scroll.backgroundColor = Colors.background
scroll.style.borderLeft = "1px solid #{Colors.border}"
scroll.style.borderLeft = "1px solid #000000"

table = new TableComponent
	width: scroll.width
	height: window.innerHeight
	superView: scroll
table.backgroundColor = ""


table.data = []

textColor = (data) ->
	return Colors.text if data.value isnt null
	return Colors.textInvisible

table.totalRows = -> @data.length
table.heightForRow = (index) -> Sizes.rowHeight
table.marginForRow = (index) -> 0
table.createRow = ->
	row = new Layer
		height: table.heightForRow()
	row.style =
		borderBottom: "1px solid #{Colors.border}"
	row.textLayer = new Layer
		x: Sizes.indent
		width: 60
		height: table.heightForRow()
		superLayer: row
	row.textLayer.style =
		font: "11px HelveticaNeue-Medium"
		color: Colors.text
		lineHeight: "#{table.heightForRow()}px"
		letterSpacing: "0.4px"
		textShadow: "0px 1px 1px rgba(0,0,0,0.50)"
		textAlign: "right"
		"-webkit-font-smoothing": "antialiased"
		"-webkit-user-select": "none"

	row.input = new InputComponent
		x: row.textLayer.maxX + Sizes.base
		width: Sizes.base * 6 
		height: row.height - (Sizes.base * 1.5)
		superLayer: row
		clip: false
		backgroundColor: "#0B0B0D"

	row.input.style =
		border: "1px solid #2A2D33"
		font: "11px HelveticaNeue-Medium"
		color: Colors.text
		letterSpacing: "0.4px"
		textShadow: "0px 1px 1px rgba(0,0,0,0.50)"
		paddingLeft: "#{Sizes.base / 2}px"
		"-webkit-font-smoothing": "antialiased"
		"-webkit-user-select": "none"

	row.input.centerY()

	_.extend row.input.inputElement.style,
		color: "#ffffff"
		# padding: "3px"


	# Input limiter
	# row.input.inputElement.onkeydown = (event) ->
	# 	if event.keyCode in [8, 190]
	# 		return true
	# 	if not String.fromCharCode(event.keyCode).match(/[0-9.]/)
	# 		return false

	row.sendValue = ->
		value = parseFloat(row.input.inputElement.value)
		return if _.isNaN(value)
		return unless app.state.selectedLayerId
		return unless app.state.selectedStateName
		bridge.send("ui:updateState", {
			layerId: app.state.selectedLayerId,
			stateName: app.state.selectedStateName,
			propertyName: row.data.name 
			value: value})

	# Change handler
	row.input.inputElement.onkeyup = ->
		row.sendValue()

	row.dragger = new Layer
		superLayer: row
		# backgroundColor: Utils.randomColor(.5)
		x: row.textLayer.x
		width: row.textLayer.width
		height: row.textLayer.height

	row.dragger.draggable.enabled = true
	row.dragger.draggable.speedX = 0

	row.dragger.on Events.DragStart, (event) ->
		row._startY = row.dragger.y
		row._startValue = parseFloat(row.input.value)

	row.dragger.on Events.DragMove, (event) ->
		deltaY = row._startY - row.dragger.y
		deltaYFraction = deltaY / (280 / (row.data.max))
		value = row._startValue + deltaYFraction
		if row.data.limit
			value = row.data.min if value < row.data.min
			value = row.data.max if value > row.data.max
		row.input.value = Utils.round(value, row.data.decimals)
		row.sendValue()

	row.dragger.on Events.DragEnd, (event) ->
		row.dragger.y = 0
	row

table.updateRow = (index, row) ->
	# return if row.data?.id == @data[index].id
	row.data = @data[index]

	value = row.data.value
	value = "0" if row.data.value is null

	row.input.inputElement._previousValue = value

	row.textLayer.html = "#{row.data.name}:"
	row.textLayer.style =
		color: textColor(row.data)

	row.input.value = "#{value}"

app.views.propertyTable = table


PROPERTIES = [
	{name: "x", type:"number", min:-1000, max:1000, decimals:0, limit:false}
	{name: "y", type:"number", min:-1000, max:1000, decimals:0, limit:false}
	{name: "z", type:"number", min:-1000, max:1000, decimals:0, limit:false}
	{name: "width", type:"number", min:0, max:1000, decimals:0, limit:true}
	{name: "height", type:"number", min:0, max:1000, decimals:0, limit:true}
	{name: "scale", type:"number", min:0, max:8, decimals:1, limit:true}
	{name: "opacity", type:"number", min:0, max:1, decimals:2, limit:true}
	{name: "rotationX", type:"number", min:0, max:360, decimals:0, limit:false}
	{name: "rotationY", type:"number", min:0, max:360, decimals:0, limit:false}
	{name: "rotationZ", type:"number", min:0, max:360, decimals:0, limit:false}
	{name: "blur", type:"number", min:0, max:100, decimals:0, limit:true}
]


getSelectedLayerData = (layerId) ->
	for item in app.views.layerTable.data
		if item.id is app.state.selectedLayerId
			return item

table.reload = ->

	if app.state.selectedLayerId
		data = getSelectedLayerData(app.state.selectedLayerId)
		state = data.states[app.state.selectedStateName]

		for item in PROPERTIES
			if state?[item.name]
				item.value = state?[item.name]
				item.stateValue = true
			else
				item.value = data[item.name]
				item.stateValue = false

	else
		for item in PROPERTIES
			item.value = null

	table.data = PROPERTIES

table.reload()
table.render()		


app.views.stateTable.on "change:selection:selection", (info) ->
	app.state.selectedStateName = info.new[0].data.name if info.new?.length
	table.reload()
	table.render()








