highlightColor = "#28AFFA"

ANIMATING_KEYS = ["x", "y", "width", "height", "scaleX", "scaleY", "scaleZ", "scale", "rotation", "rotationX", "rotationY", "rotationZ", "skew", "skewX", "skewY"]

CONFIG =
	borderWidth: 2

# We include th
canvasScaleX = (layer) ->
	scale = layer.scale * layer.scaleX
	for superLayer in layer.superLayers(context=true)
		scale = scale * superLayer.scale * superLayer.scaleX
	return scale

canvasScaleY = (layer) ->
	scale = layer.scale * layer.scaleY
	for superLayer in layer.superLayers(context=true)
		scale = scale * superLayer.scale * superLayer.scaleY
	return scale

screenScaledFrame = (layer) ->

	# TODO: Scroll position

	frame =
		x: 0
		y: 0
		width:  layer.width  * canvasScaleX(layer)
		height: layer.height * canvasScaleY(layer)
	
	layers = layer.superLayers(context=true)
	layers.push(layer)
	layers.reverse()
	
	for superLayer in layers
		factorX = if superLayer._superOrParentLayer() then canvasScaleX(superLayer._superOrParentLayer()) else 1
		factorY = if superLayer._superOrParentLayer() then canvasScaleY(superLayer._superOrParentLayer()) else 1
		layerScaledFrame = scaledFrame(superLayer)
		frame.x += layerScaledFrame.x * factorX
		frame.y += layerScaledFrame.y * factorY

	return frame

scaledFrame = (layer) ->

	# Get the scaled frame for a layer, taking into account 
	# the transform origins.

	frame = layer.frame
	scaleX = layer.scale * layer.scaleX
	scaleY = layer.scale * layer.scaleY

	frame.width  *= scaleX
	frame.height *= scaleY
	frame.x += (1 - scaleX) * layer.originX * layer.width
	frame.y += (1 - scaleY) * layer.originY * layer.height
	
	return frame


class exports.HighlightComponent
	constructor: ->
		ctx = new Framer.Context name:"Highlight"
		ctx.run =>
			@layer = new Layer
			@info = new Layer
		@layer.style =
			border: "#{CONFIG.borderWidth}px solid #{highlightColor}"
			zIndex:10000
		@layer.backgroundColor = "rgba(40,175,250,0.2)"
		@info.style =
			font: "bold 11px HelveticaNeue"
			zIndex: 10000
			textAlign: "center"
			letterSpacing: ".4px"
		@info.color = "white"
		@info.backgroundColor = "transparent"
		@info.textElement = document.createElement("div")
		_.extend @info.textElement.style,
			color: "#FFFFFF"
			display: "inline-block"
			backgroundColor: "rgba(40,175,250,1)"
			borderRadius: "3px"
			padding: "5px 8px 5px 8px"
			textShadow: "0px 1px 0px rgba(0,0,0,0.1)"
		@info._element.appendChild(@info.textElement)

		@layer.visible = false
		@info.visible = false

		@unhighlight()
	highlight: (layer) =>
		return if @current is layer
		@current = layer
		@update()
		for p in ANIMATING_KEYS
			@current.on("change:#{p}", @update)

	unhighlight: ->
		return unless @current
		for p in ANIMATING_KEYS
			@current.off("change:#{p}", @update)
		@current = null
		@layer.visible = false
		@info.visible = false

	update: =>
		return unless @layer
		return unless @current

		# check if the Framer version supports bounding frames for a more precise result
		if Utils.boundingFrame
			currentFrame = Utils.boundingFrame(@current, true)
			screenFrame = Utils.boundingFrame(@current, false)
			@info.textElement.textContent = "
				x: #{screenFrame.x.toFixed(1)}, 
				y: #{screenFrame.y.toFixed(1)}, 
				width: #{screenFrame.width.toFixed(1)}, 
				height: #{screenFrame.height.toFixed(1)}"
		else
			currentFrame = screenScaledFrame(@current)
			@layer.rotation = @current.rotation
			@info.textElement.textContent = "
				x: #{@current.x.toFixed(1)}, 
				y: #{@current.y.toFixed(1)}, 
				width: #{@current.width.toFixed(1)}, 
				height: #{@current.height.toFixed(1)}"

		currentFrame.x -= CONFIG.borderWidth
		currentFrame.y -= CONFIG.borderWidth
		currentFrame.width  += 2 * CONFIG.borderWidth
		currentFrame.height += 2 * CONFIG.borderWidth

		@layer.visible = true
		@layer.frame = currentFrame

		canvasFrame = Framer.Canvas?.frame

		if not canvasFrame
			canvasFrame =
				width:  Screen.width
				height: Screen.height

		margin = 12

		@info.visible = true
		@info.frame = currentFrame
		@info.width = 320
		@info.height = 24

		@info.style.textAlign = "center"
		midYPos = @layer.midY 
		midXPos = @layer.midX
		infoOnTopOrBottom = true

		if @layer.y < canvasFrame.height - (@info.height + margin) && @layer.maxY > (@info.height + margin)
			if midXPos > canvasFrame.width - ((@info.width / 2) + margin)
				@info.style.textAlign = "right"
				@info.maxX = Math.min(@layer.x - margin, canvasFrame.width - margin)
				infoOnTopOrBottom = false
			else if midXPos < ((@info.width / 2) + margin)
				@info.style.textAlign = "left"
				@info.x = Math.max(@layer.maxX + margin, margin)
				infoOnTopOrBottom = false
			if !infoOnTopOrBottom
				@info.midY = Math.max(Math.min(midYPos, canvasFrame.height - ((@info.height + margin) - margin) ), (@info.height + margin) - margin)

		if infoOnTopOrBottom
			yPos = @layer.maxY + margin

			if yPos > canvasFrame.height - (@info.height + margin)
				yPos = @layer.y - (@info.height + margin)
				if yPos > canvasFrame.height - (@info.height + margin)
					yPos = canvasFrame.height - (@info.height + margin)
			else if yPos < margin
					yPos = margin

			if midXPos > canvasFrame.width - ((@info.width / 2) + margin)
				midXPos = canvasFrame.width - ((@info.width / 2) + margin)
				@info.style.textAlign = "right"

			else if midXPos < ((@info.width / 2) + margin)
				midXPos = ((@info.width / 2) + margin)
				@info.style.textAlign = "left"

			@info.midX = midXPos
			@info.y = yPos


		@info.x = Math.max(margin, @info.x)
		@info.maxX = Math.min(canvasFrame.width - margin, @info.maxX)
		@info.y = Math.max(margin, @info.y)
		@info.maxY = Math.min(canvasFrame.height - margin, @info.maxY)

		@info.pixelAlign()
		
