import os
import sys
import mmap
import re
import json

if len(sys.argv) < 2:
	sys.exit("Usage: syslog.py <output-path.json>")

OUT_PATH = sys.argv[1]
LOG_PATH = "/var/log/system.log"
LINES = 500

def tail(filename, n):
	"""Returns last n lines from the filename. No exception handling"""
	size = os.path.getsize(filename)
	with open(filename, "rb") as f:
		# for Windows the mmap parameters are different
		fm = mmap.mmap(f.fileno(), 0, mmap.MAP_SHARED, mmap.PROT_READ)
		try:
			for i in xrange(size - 1, -1, -1):
				if fm[i] == '\n':
					n -= 1
					if n == -1:
						break
			return fm[i + 1 if i else 0:]
		finally:
			fm.close()

# https://regex101.com/r/nI9eK5/3
p = re.compile(ur'([a-zA-Z]{3}\s+\d{1,2}\s+\d{1,2}:\d{2}:\d{2})\s+([a-zA-Z-0-9]+)\s+([a-zA-Z-0-9\(\)\. ]+)\[(\d+)]: (.+?(?=\n([a-zA-Z]{3}\s+\d{1,2}\s+\d{1,2}:\d{2}:\d{2})))', re.MULTILINE | re.DOTALL)
text = tail(LOG_PATH, LINES)

data = []
 
for match in re.findall(p, text):
	data.append({
		"date": match[0],
		"hostname": match[1],
		"name": match[2],
		"pid": int(match[3]),
		"message": match[4].strip()
	})


with open(OUT_PATH, "w") as f:
	f.write(json.dumps(data, sort_keys=True, indent=4))
